@@ -42,6 +42,10 @@ class ShouldClose(Exception):
     pass
 
 
+class EndOfStream(Exception):
+    pass
+
+
 class WebSocketSession:
     """
     Sync context manager representing an opened WebSocket session.
@@ -53,8 +57,8 @@ class WebSocketSession:
             The webSocket handshake response.
     """
 
-    subprotocol: typing.Optional[str]
-    response: typing.Optional[httpx.Response]
+    subprotocol: str | None
+    response: httpx.Response | None
 
     def __init__(
         self,
@@ -62,13 +66,11 @@ class WebSocketSession:
         *,
         max_message_size_bytes: int = DEFAULT_MAX_MESSAGE_SIZE_BYTES,
         queue_size: int = DEFAULT_QUEUE_SIZE,
-        keepalive_ping_interval_seconds: typing.Optional[
-            float
-        ] = DEFAULT_KEEPALIVE_PING_INTERVAL_SECONDS,
-        keepalive_ping_timeout_seconds: typing.Optional[
-            float
-        ] = DEFAULT_KEEPALIVE_PING_TIMEOUT_SECONDS,
-        response: typing.Optional[httpx.Response] = None,
+        keepalive_ping_interval_seconds: float
+        | None = DEFAULT_KEEPALIVE_PING_INTERVAL_SECONDS,
+        keepalive_ping_timeout_seconds: float
+        | None = DEFAULT_KEEPALIVE_PING_TIMEOUT_SECONDS,
+        response: httpx.Response | None = None,
     ) -> None:
         self.stream = stream
         self.connection = wsproto.connection.Connection(wsproto.ConnectionType.CLIENT)
@@ -78,15 +80,15 @@ class WebSocketSession:
         else:
             self.subprotocol = None
 
-        self._events: queue.Queue[
-            typing.Union[wsproto.events.Event, HTTPXWSException]
-        ] = queue.Queue(queue_size)
+        self._events: queue.Queue[wsproto.events.Event | HTTPXWSException] = (
+            queue.Queue(queue_size)
+        )
 
         self._ping_manager = PingManager()
         self._should_close = threading.Event()
         self._write_lock: threading.Lock = threading.Lock()
-        self._should_close_task: typing.Optional[concurrent.futures.Future[bool]] = None
-        self._executor: typing.Optional[concurrent.futures.ThreadPoolExecutor] = None
+        self._should_close_task: concurrent.futures.Future[bool] | None = None
+        self._executor: concurrent.futures.ThreadPoolExecutor | None = None
 
         self._max_message_size_bytes = max_message_size_bytes
         self._queue_size = queue_size
@@ -110,7 +112,7 @@ class WebSocketSession:
         )
         self._background_receive_task.start()
 
-        self._background_keepalive_ping_task: typing.Optional[threading.Thread] = None
+        self._background_keepalive_ping_task: threading.Thread | None = None
         if self._keepalive_ping_interval_seconds is not None:
             self._background_keepalive_ping_task = threading.Thread(
                 target=self._background_keepalive_ping,
@@ -246,7 +248,7 @@ class WebSocketSession:
         else:
             self.send_bytes(serialized_data.encode("utf-8"))
 
-    def receive(self, timeout: typing.Optional[float] = None) -> wsproto.events.Event:
+    def receive(self, timeout: float | None = None) -> wsproto.events.Event:
         """
         Receive an event from the server.
 
@@ -292,7 +294,7 @@ class WebSocketSession:
             raise WebSocketDisconnect(event.code, event.reason)
         return event
 
-    def receive_text(self, timeout: typing.Optional[float] = None) -> str:
+    def receive_text(self, timeout: float | None = None) -> str:
         """
         Receive text from the server.
 
@@ -332,7 +334,7 @@ class WebSocketSession:
             return event.data
         raise WebSocketInvalidTypeReceived(event)
 
-    def receive_bytes(self, timeout: typing.Optional[float] = None) -> bytes:
+    def receive_bytes(self, timeout: float | None = None) -> bytes:
         """
         Receive bytes from the server.
 
@@ -373,7 +375,7 @@ class WebSocketSession:
         raise WebSocketInvalidTypeReceived(event)
 
     def receive_json(
-        self, timeout: typing.Optional[float] = None, mode: JSONMode = "text"
+        self, timeout: float | None = None, mode: JSONMode = "text"
     ) -> typing.Any:
         """
         Receive JSON data from the server.
@@ -415,14 +417,14 @@ class WebSocketSession:
                     print("Connection closed")
         """
         assert mode in ["text", "binary"]
-        data: typing.Union[str, bytes]
+        data: str | bytes
         if mode == "text":
             data = self.receive_text(timeout)
         elif mode == "binary":
             data = self.receive_bytes(timeout)
         return json.loads(data)
 
-    def close(self, code: int = 1000, reason: typing.Optional[str] = None):
+    def close(self, code: int = 1000, reason: str | None = None):
         """
         Close the WebSocket session.
 
@@ -472,10 +474,10 @@ class WebSocketSession:
         Args:
             max_bytes: The maximum chunk size to read at each iteration.
         """
-        partial_message_buffer: typing.Union[str, bytes, None] = None
+        partial_message_buffer: str | bytes | None = None
         try:
             while not self._should_close.is_set():
-                data = self._wait_until_closed(self.stream.read, max_bytes)
+                data = self._wait_until_closed(self._read_stream, max_bytes)
                 self.connection.receive_data(data)
                 for event in self.connection.events():
                     if isinstance(event, wsproto.events.Ping):
@@ -508,14 +510,14 @@ class WebSocketSession:
                             self._events.put(full_message_event)
                         continue
                     self._events.put(event)
-        except (httpcore.ReadError, httpcore.WriteError):
+        except (httpcore.ReadError, httpcore.WriteError, EndOfStream):
             self.close(CloseReason.INTERNAL_ERROR, "Stream error")
             self._events.put(WebSocketNetworkError())
         except ShouldClose:
             pass
 
     def _background_keepalive_ping(
-        self, interval_seconds: float, timeout_seconds: typing.Optional[float] = None
+        self, interval_seconds: float, timeout_seconds: float | None = None
     ) -> None:
         try:
             while not self._should_close.is_set():
@@ -554,6 +556,12 @@ class WebSocketSession:
             result = todo_task.result()
         return result
 
+    def _read_stream(self, max_bytes: int) -> bytes:
+        data = self.stream.read(max_bytes)
+        if data == b"":
+            raise EndOfStream()
+        return data
+
 
 class AsyncWebSocketSession:
     """
@@ -566,14 +574,10 @@ class AsyncWebSocketSession:
             The webSocket handshake response.
     """
 
-    subprotocol: typing.Optional[str]
-    response: typing.Optional[httpx.Response]
-    _send_event: MemoryObjectSendStream[
-        typing.Union[wsproto.events.Event, HTTPXWSException]
-    ]
-    _receive_event: MemoryObjectReceiveStream[
-        typing.Union[wsproto.events.Event, HTTPXWSException]
-    ]
+    subprotocol: str | None
+    response: httpx.Response | None
+    _send_event: MemoryObjectSendStream[wsproto.events.Event | HTTPXWSException]
+    _receive_event: MemoryObjectReceiveStream[wsproto.events.Event | HTTPXWSException]
 
     def __init__(
         self,
@@ -581,13 +585,11 @@ class AsyncWebSocketSession:
         *,
         max_message_size_bytes: int = DEFAULT_MAX_MESSAGE_SIZE_BYTES,
         queue_size: int = DEFAULT_QUEUE_SIZE,
-        keepalive_ping_interval_seconds: typing.Optional[
-            float
-        ] = DEFAULT_KEEPALIVE_PING_INTERVAL_SECONDS,
-        keepalive_ping_timeout_seconds: typing.Optional[
-            float
-        ] = DEFAULT_KEEPALIVE_PING_TIMEOUT_SECONDS,
-        response: typing.Optional[httpx.Response] = None,
+        keepalive_ping_interval_seconds: float
+        | None = DEFAULT_KEEPALIVE_PING_INTERVAL_SECONDS,
+        keepalive_ping_timeout_seconds: float
+        | None = DEFAULT_KEEPALIVE_PING_TIMEOUT_SECONDS,
+        response: httpx.Response | None = None,
     ) -> None:
         self.stream = stream
         self.connection = wsproto.connection.Connection(wsproto.ConnectionType.CLIENT)
@@ -615,7 +617,7 @@ class AsyncWebSocketSession:
     async def __aenter__(self) -> "AsyncWebSocketSession":
         async with contextlib.AsyncExitStack() as exit_stack:
             self._send_event, self._receive_event = anyio.create_memory_object_stream[
-                typing.Union[wsproto.events.Event, HTTPXWSException]
+                wsproto.events.Event | HTTPXWSException
             ]()
             exit_stack.enter_context(self._send_event)
             exit_stack.enter_context(self._receive_event)
@@ -641,9 +643,9 @@ class AsyncWebSocketSession:
 
     async def __aexit__(
         self,
-        exc_type: typing.Optional[type[BaseException]],
-        exc: typing.Optional[BaseException],
-        tb: typing.Optional[TracebackType],
+        exc_type: type[BaseException] | None,
+        exc: BaseException | None,
+        tb: TracebackType | None,
     ) -> None:
         await self._exit_stack.aclose()
 
@@ -764,9 +766,7 @@ class AsyncWebSocketSession:
         else:
             await self.send_bytes(serialized_data.encode("utf-8"))
 
-    async def receive(
-        self, timeout: typing.Optional[float] = None
-    ) -> wsproto.events.Event:
+    async def receive(self, timeout: float | None = None) -> wsproto.events.Event:
         """
         Receive an event from the server.
 
@@ -813,7 +813,7 @@ class AsyncWebSocketSession:
             raise WebSocketDisconnect(event.code, event.reason)
         return event
 
-    async def receive_text(self, timeout: typing.Optional[float] = None) -> str:
+    async def receive_text(self, timeout: float | None = None) -> str:
         """
         Receive text from the server.
 
@@ -853,7 +853,7 @@ class AsyncWebSocketSession:
             return event.data
         raise WebSocketInvalidTypeReceived(event)
 
-    async def receive_bytes(self, timeout: typing.Optional[float] = None) -> bytes:
+    async def receive_bytes(self, timeout: float | None = None) -> bytes:
         """
         Receive bytes from the server.
 
@@ -894,7 +894,7 @@ class AsyncWebSocketSession:
         raise WebSocketInvalidTypeReceived(event)
 
     async def receive_json(
-        self, timeout: typing.Optional[float] = None, mode: JSONMode = "text"
+        self, timeout: float | None = None, mode: JSONMode = "text"
     ) -> typing.Any:
         """
         Receive JSON data from the server.
@@ -936,14 +936,14 @@ class AsyncWebSocketSession:
                     print("Connection closed")
         """
         assert mode in ["text", "binary"]
-        data: typing.Union[str, bytes]
+        data: str | bytes
         if mode == "text":
             data = await self.receive_text(timeout)
         elif mode == "binary":
             data = await self.receive_bytes(timeout)
         return json.loads(data)
 
-    async def close(self, code: int = 1000, reason: typing.Optional[str] = None):
+    async def close(self, code: int = 1000, reason: str | None = None):
         """
         Close the WebSocket session.
 
@@ -991,10 +991,10 @@ class AsyncWebSocketSession:
         Args:
             max_bytes: The maximum chunk size to read at each iteration.
         """
-        partial_message_buffer: typing.Union[str, bytes, None] = None
+        partial_message_buffer: str | bytes | None = None
         try:
             while not self._should_close.is_set():
-                data = await self.stream.read(max_bytes=max_bytes)
+                data = await self._read_stream(max_bytes)
                 self.connection.receive_data(data)
                 for event in self.connection.events():
                     if isinstance(event, wsproto.events.Ping):
@@ -1027,12 +1027,12 @@ class AsyncWebSocketSession:
                             await self._send_event.send(full_message_event)
                         continue
                     await self._send_event.send(event)
-        except (httpcore.ReadError, httpcore.WriteError):
+        except (httpcore.ReadError, httpcore.WriteError, EndOfStream):
             await self.close(CloseReason.INTERNAL_ERROR, "Stream error")
             await self._send_event.send(WebSocketNetworkError())
 
     async def _background_keepalive_ping(
-        self, interval_seconds: float, timeout_seconds: typing.Optional[float] = None
+        self, interval_seconds: float, timeout_seconds: float | None = None
     ) -> None:
         while not self._should_close.is_set():
             await anyio.sleep(interval_seconds)
@@ -1047,9 +1047,15 @@ class AsyncWebSocketSession:
                     )
                     await self._send_event.send(WebSocketNetworkError())
 
+    async def _read_stream(self, max_bytes: int) -> bytes:
+        data = await self.stream.read(max_bytes)
+        if data == b"":
+            raise EndOfStream()
+        return data
+
 
 def _get_headers(
-    subprotocols: typing.Optional[list[str]],
+    subprotocols: list[str] | None,
 ) -> dict[str, typing.Any]:
     headers = {
         "connection": "upgrade",
@@ -1062,56 +1068,126 @@ def _get_headers(
     return headers
 
 
-@contextlib.contextmanager
-def _connect_ws(
-    url: str,
-    client: httpx.Client,
-    *,
-    max_message_size_bytes: int = DEFAULT_MAX_MESSAGE_SIZE_BYTES,
-    queue_size: int = DEFAULT_QUEUE_SIZE,
-    keepalive_ping_interval_seconds: typing.Optional[
-        float
-    ] = DEFAULT_KEEPALIVE_PING_INTERVAL_SECONDS,
-    keepalive_ping_timeout_seconds: typing.Optional[
-        float
-    ] = DEFAULT_KEEPALIVE_PING_TIMEOUT_SECONDS,
-    subprotocols: typing.Optional[list[str]] = None,
-    session_class: type[SyncSession] = WebSocketSession,  # type: ignore[assignment]
-    **kwargs: typing.Any,
-) -> typing.Generator[SyncSession, None, None]:
-    headers = kwargs.pop("headers", {})
-    headers.update(_get_headers(subprotocols))
+class WebSocketClient(typing.Generic[SyncSession]):
+    """
+    An sync WebSocket client.
 
-    with client.stream("GET", url, headers=headers, **kwargs) as response:
-        if response.status_code != 101:
-            raise WebSocketUpgradeError(response)
+    This class provides an API for connecting to WebSocket.
 
-        session = session_class(
-            response.extensions["network_stream"],
-            max_message_size_bytes=max_message_size_bytes,
-            queue_size=queue_size,
-            keepalive_ping_interval_seconds=keepalive_ping_interval_seconds,
-            keepalive_ping_timeout_seconds=keepalive_ping_timeout_seconds,
-            response=response,
-        )
-        with session:
-            yield session
+    Attributes:
+        client:
+            HTTPX client to use.
+            If not provided, a default one will be initialized.
+        max_message_size_bytes:
+            Message size in bytes to receive from the server.
+            Defaults to 65 KiB.
+        queue_size:
+            Size of the queue where the received messages will be held
+            until they are consumed.
+            If the queue is full, the client will stop receive messages
+            from the server until the queue has room available.
+            Defaults to 512.
+        keepalive_ping_interval_seconds:
+            Interval at which the client will automatically send a Ping event
+            to keep the connection alive. Set it to `None` to disable this mechanism.
+            Defaults to 20 seconds.
+        keepalive_ping_timeout_seconds:
+            Maximum delay the client will wait for an answer to its Ping event.
+            If the delay is exceeded,
+            [WebSocketNetworkError][httpx_ws.WebSocketNetworkError]
+            will be raised and the connection closed.
+            Defaults to 20 seconds.
+        session_class:
+            The session class to use.
+            Defaults to [WebSocketSession][httpx_ws.WebSocketSession].
+    """
+
+    def __init__(
+        self,
+        client: httpx.Client,
+        *,
+        max_message_size_bytes: int = DEFAULT_MAX_MESSAGE_SIZE_BYTES,
+        queue_size: int = DEFAULT_QUEUE_SIZE,
+        keepalive_ping_interval_seconds: float
+        | None = DEFAULT_KEEPALIVE_PING_INTERVAL_SECONDS,
+        keepalive_ping_timeout_seconds: float
+        | None = DEFAULT_KEEPALIVE_PING_TIMEOUT_SECONDS,
+        session_class: type[SyncSession] = WebSocketSession,  # type: ignore[assignment]
+    ) -> None:
+        self.client = client
+        self.max_message_size_bytes = max_message_size_bytes
+        self.queue_size = queue_size
+        self.keepalive_ping_interval_seconds = keepalive_ping_interval_seconds
+        self.keepalive_ping_timeout_seconds = keepalive_ping_timeout_seconds
+        self.session_class = session_class
+
+    @contextlib.contextmanager
+    def connect(
+        self,
+        url: str,
+        subprotocols: list[str] | None = None,
+        **kwargs: typing.Any,
+    ) -> typing.Generator[SyncSession, None, None]:
+        """
+        Start a sync WebSocket session.
+
+        It returns a context manager that'll automatically
+        call [close()][httpx_ws.WebSocketSession.close] when exiting.
+
+        Args:
+            url: The WebSocket URL.
+            subprotocols:
+                Optional list of subprotocols to negotiate with the server.
+            **kwargs:
+                Additional keyword arguments that will be passed to
+                the [HTTPX stream()](https://www.python-httpx.org/api/#request) method.
+
+        Returns:
+            A [context manager][contextlib.AbstractContextManager]
+                for [WebSocketSession][httpx_ws.WebSocketSession].
+
+        Examples:
+            Initialize the client and connect to a WebSocket.
+
+                with httpx.Client() as client:
+                    ws_client = WebSocketClient(client)
+                    with ws_client.connect("http://localhost:8000/ws") as ws:
+                        message = ws.receive_text()
+                        print(message)
+                        ws.send_text("Hello!")
+        """
+
+        headers = kwargs.pop("headers", {})
+        headers.update(_get_headers(subprotocols))
+
+        with self.client.stream("GET", url, headers=headers, **kwargs) as response:
+            if response.status_code != 101:
+                raise WebSocketUpgradeError(response)
+
+            session = self.session_class(
+                response.extensions["network_stream"],
+                max_message_size_bytes=self.max_message_size_bytes,
+                queue_size=self.queue_size,
+                keepalive_ping_interval_seconds=self.keepalive_ping_interval_seconds,
+                keepalive_ping_timeout_seconds=self.keepalive_ping_timeout_seconds,
+                response=response,
+            )
+            with session:
+                yield session
 
 
 @contextlib.contextmanager
 def connect_ws(
     url: str,
-    client: typing.Optional[httpx.Client] = None,
+    client: httpx.Client | None = None,
     *,
     max_message_size_bytes: int = DEFAULT_MAX_MESSAGE_SIZE_BYTES,
     queue_size: int = DEFAULT_QUEUE_SIZE,
-    keepalive_ping_interval_seconds: typing.Optional[
-        float
-    ] = DEFAULT_KEEPALIVE_PING_INTERVAL_SECONDS,
-    keepalive_ping_timeout_seconds: typing.Optional[
-        float
-    ] = DEFAULT_KEEPALIVE_PING_TIMEOUT_SECONDS,
-    subprotocols: typing.Optional[list[str]] = None,
+    keepalive_ping_interval_seconds: float
+    | None = DEFAULT_KEEPALIVE_PING_INTERVAL_SECONDS,
+    keepalive_ping_timeout_seconds: float
+    | None = DEFAULT_KEEPALIVE_PING_TIMEOUT_SECONDS,
+    subprotocols: list[str] | None = None,
     session_class: type[SyncSession] = WebSocketSession,  # type: ignore[assignment]
     **kwargs: typing.Any,
 ) -> typing.Generator[SyncSession, None, None]:
@@ -1146,7 +1222,7 @@ def connect_ws(
             will be raised and the connection closed.
             Defaults to 20 seconds.
         subprotocols:
-            Optional list of suprotocols to negotiate with the server.
+            Optional list of subprotocols to negotiate with the server.
         session_class:
             The session class to use.
             Defaults to [WebSocketSession][httpx_ws.WebSocketSession].
@@ -1176,83 +1252,152 @@ def connect_ws(
     """
     if client is None:
         with httpx.Client() as client:
-            with _connect_ws(
-                url,
+            ws_client = WebSocketClient(
                 client=client,
                 max_message_size_bytes=max_message_size_bytes,
                 queue_size=queue_size,
                 keepalive_ping_interval_seconds=keepalive_ping_interval_seconds,
                 keepalive_ping_timeout_seconds=keepalive_ping_timeout_seconds,
-                subprotocols=subprotocols,
                 session_class=session_class,
-                **kwargs,
+            )
+            with ws_client.connect(
+                url, subprotocols=subprotocols, **kwargs
             ) as websocket:
                 yield websocket
     else:
-        with _connect_ws(
-            url,
+        ws_client = WebSocketClient(
             client=client,
             max_message_size_bytes=max_message_size_bytes,
             queue_size=queue_size,
             keepalive_ping_interval_seconds=keepalive_ping_interval_seconds,
             keepalive_ping_timeout_seconds=keepalive_ping_timeout_seconds,
-            subprotocols=subprotocols,
             session_class=session_class,
-            **kwargs,
-        ) as websocket:
+        )
+        with ws_client.connect(url, subprotocols=subprotocols, **kwargs) as websocket:
             yield websocket
 
 
-@contextlib.asynccontextmanager
-async def _aconnect_ws(
-    url: str,
-    client: httpx.AsyncClient,
-    *,
-    max_message_size_bytes: int = DEFAULT_MAX_MESSAGE_SIZE_BYTES,
-    queue_size: int = DEFAULT_QUEUE_SIZE,
-    keepalive_ping_interval_seconds: typing.Optional[
-        float
-    ] = DEFAULT_KEEPALIVE_PING_INTERVAL_SECONDS,
-    keepalive_ping_timeout_seconds: typing.Optional[
-        float
-    ] = DEFAULT_KEEPALIVE_PING_TIMEOUT_SECONDS,
-    subprotocols: typing.Optional[list[str]] = None,
-    session_class: type[AsyncSession] = AsyncWebSocketSession,  # type: ignore[assignment]
-    **kwargs: typing.Any,
-) -> typing.AsyncGenerator[AsyncSession, None]:
-    headers = kwargs.pop("headers", {})
-    headers.update(_get_headers(subprotocols))
+class AsyncWebSocketClient(typing.Generic[AsyncSession]):
+    """
+    An async WebSocket client.
 
-    async with client.stream("GET", url, headers=headers, **kwargs) as response:
-        if response.status_code != 101:
-            raise WebSocketUpgradeError(response)
+    This class provides an API for connecting to WebSocket.
 
-        session = session_class(
-            response.extensions["network_stream"],
-            max_message_size_bytes=max_message_size_bytes,
-            queue_size=queue_size,
-            keepalive_ping_interval_seconds=keepalive_ping_interval_seconds,
-            keepalive_ping_timeout_seconds=keepalive_ping_timeout_seconds,
-            response=response,
-        )
-        async with session:
-            yield session
+    Attributes:
+        client:
+            HTTPX client to use.
+            If not provided, a default one will be initialized.
+        max_message_size_bytes:
+            Message size in bytes to receive from the server.
+            Defaults to 65 KiB.
+        queue_size:
+            Size of the queue where the received messages will be held
+            until they are consumed.
+            If the queue is full, the client will stop receive messages
+            from the server until the queue has room available.
+            Defaults to 512.
+        keepalive_ping_interval_seconds:
+            Interval at which the client will automatically send a Ping event
+            to keep the connection alive. Set it to `None` to disable this mechanism.
+            Defaults to 20 seconds.
+        keepalive_ping_timeout_seconds:
+            Maximum delay the client will wait for an answer to its Ping event.
+            If the delay is exceeded,
+            [WebSocketNetworkError][httpx_ws.WebSocketNetworkError]
+            will be raised and the connection closed.
+            Defaults to 20 seconds.
+        session_class:
+            The session class to use.
+            Defaults to [AsyncWebSocketSession][httpx_ws.AsyncWebSocketSession].
+    """
+
+    def __init__(
+        self,
+        client: httpx.AsyncClient,
+        *,
+        max_message_size_bytes: int = DEFAULT_MAX_MESSAGE_SIZE_BYTES,
+        queue_size: int = DEFAULT_QUEUE_SIZE,
+        keepalive_ping_interval_seconds: float
+        | None = DEFAULT_KEEPALIVE_PING_INTERVAL_SECONDS,
+        keepalive_ping_timeout_seconds: float
+        | None = DEFAULT_KEEPALIVE_PING_TIMEOUT_SECONDS,
+        session_class: type[AsyncSession] = AsyncWebSocketSession,  # type: ignore[assignment]
+    ) -> None:
+        self.client = client
+        self.max_message_size_bytes = max_message_size_bytes
+        self.queue_size = queue_size
+        self.keepalive_ping_interval_seconds = keepalive_ping_interval_seconds
+        self.keepalive_ping_timeout_seconds = keepalive_ping_timeout_seconds
+        self.session_class = session_class
+
+    @contextlib.asynccontextmanager
+    async def connect(
+        self,
+        url: str,
+        subprotocols: list[str] | None = None,
+        **kwargs: typing.Any,
+    ) -> typing.AsyncGenerator[AsyncSession, None]:
+        """
+        Start an async WebSocket session.
+
+        It returns an async context manager that'll automatically
+        call [close()][httpx_ws.AsyncWebSocketSession.close] when exiting.
+
+        Args:
+            url: The WebSocket URL.
+            subprotocols:
+                Optional list of subprotocols to negotiate with the server.
+            **kwargs:
+                Additional keyword arguments that will be passed to
+                the [HTTPX stream()](https://www.python-httpx.org/api/#request) method.
+
+        Returns:
+            An [async context manager][contextlib.AbstractAsyncContextManager]
+                for [AsyncWebSocketSession][httpx_ws.AsyncWebSocketSession].
+
+        Examples:
+            Initialize the client and connect to a WebSocket.
+
+                async with httpx.AsyncClient() as client:
+                    ws_client = AsyncWebSocketClient(client)
+                    async with ws_client.connect("http://localhost:8000/ws") as ws:
+                        message = await ws.receive_text()
+                        print(message)
+                        await ws.send_text("Hello!")
+        """
+        headers = kwargs.pop("headers", {})
+        headers.update(_get_headers(subprotocols))
+
+        async with self.client.stream(
+            "GET", url, headers=headers, **kwargs
+        ) as response:
+            if response.status_code != 101:
+                raise WebSocketUpgradeError(response)
+
+            session = self.session_class(
+                response.extensions["network_stream"],
+                max_message_size_bytes=self.max_message_size_bytes,
+                queue_size=self.queue_size,
+                keepalive_ping_interval_seconds=self.keepalive_ping_interval_seconds,
+                keepalive_ping_timeout_seconds=self.keepalive_ping_timeout_seconds,
+                response=response,
+            )
+            async with session:
+                yield session
 
 
 @contextlib.asynccontextmanager
 async def aconnect_ws(
     url: str,
-    client: typing.Optional[httpx.AsyncClient] = None,
+    client: httpx.AsyncClient | None = None,
     *,
     max_message_size_bytes: int = DEFAULT_MAX_MESSAGE_SIZE_BYTES,
     queue_size: int = DEFAULT_QUEUE_SIZE,
-    keepalive_ping_interval_seconds: typing.Optional[
-        float
-    ] = DEFAULT_KEEPALIVE_PING_INTERVAL_SECONDS,
-    keepalive_ping_timeout_seconds: typing.Optional[
-        float
-    ] = DEFAULT_KEEPALIVE_PING_TIMEOUT_SECONDS,
-    subprotocols: typing.Optional[list[str]] = None,
+    keepalive_ping_interval_seconds: float
+    | None = DEFAULT_KEEPALIVE_PING_INTERVAL_SECONDS,
+    keepalive_ping_timeout_seconds: float
+    | None = DEFAULT_KEEPALIVE_PING_TIMEOUT_SECONDS,
+    subprotocols: list[str] | None = None,
     session_class: type[AsyncSession] = AsyncWebSocketSession,  # type: ignore[assignment]
     **kwargs: typing.Any,
 ) -> typing.AsyncGenerator[AsyncSession, None]:
@@ -1287,7 +1432,7 @@ async def aconnect_ws(
             will be raised and the connection closed.
             Defaults to 20 seconds.
         subprotocols:
-            Optional list of suprotocols to negotiate with the server.
+            Optional list of subprotocols to negotiate with the server.
         session_class:
             The session class to use.
             Defaults to [AsyncWebSocketSession][httpx_ws.AsyncWebSocketSession].
@@ -1317,28 +1462,28 @@ async def aconnect_ws(
     """
     if client is None:
         async with httpx.AsyncClient() as client:
-            async with _aconnect_ws(
-                url,
+            ws_client = AsyncWebSocketClient(
                 client=client,
                 max_message_size_bytes=max_message_size_bytes,
                 queue_size=queue_size,
                 keepalive_ping_interval_seconds=keepalive_ping_interval_seconds,
                 keepalive_ping_timeout_seconds=keepalive_ping_timeout_seconds,
-                subprotocols=subprotocols,
                 session_class=session_class,
-                **kwargs,
+            )
+            async with ws_client.connect(
+                url, subprotocols=subprotocols, **kwargs
             ) as websocket:
                 yield websocket
     else:
-        async with _aconnect_ws(
-            url,
+        ws_client = AsyncWebSocketClient(
             client=client,
             max_message_size_bytes=max_message_size_bytes,
             queue_size=queue_size,
             keepalive_ping_interval_seconds=keepalive_ping_interval_seconds,
             keepalive_ping_timeout_seconds=keepalive_ping_timeout_seconds,
-            subprotocols=subprotocols,
             session_class=session_class,
-            **kwargs,
+        )
+        async with ws_client.connect(
+            url, subprotocols=subprotocols, **kwargs
         ) as websocket:
             yield websocket
