@@ -1,6 +1,5 @@
 import secrets
 import threading
-import typing
 
 import anyio
 
@@ -14,15 +13,13 @@ class PingManager(PingManagerBase):
     def __init__(self) -> None:
         self._pings: dict[bytes, threading.Event] = {}
 
-    def create(
-        self, ping_id: typing.Optional[bytes] = None
-    ) -> tuple[bytes, threading.Event]:
+    def create(self, ping_id: bytes | None = None) -> tuple[bytes, threading.Event]:
         ping_id = self._generate_id() if not ping_id else ping_id
         event = threading.Event()
         self._pings[ping_id] = event
         return ping_id, event
 
-    def ack(self, ping_id: typing.Union[bytes, bytearray]):
+    def ack(self, ping_id: bytes | bytearray):
         event = self._pings.pop(bytes(ping_id))
         event.set()
 
@@ -31,14 +28,12 @@ class AsyncPingManager(PingManagerBase):
     def __init__(self) -> None:
         self._pings: dict[bytes, anyio.Event] = {}
 
-    def create(
-        self, ping_id: typing.Optional[bytes] = None
-    ) -> tuple[bytes, anyio.Event]:
+    def create(self, ping_id: bytes | None = None) -> tuple[bytes, anyio.Event]:
         ping_id = self._generate_id() if not ping_id else ping_id
         event = anyio.Event()
         self._pings[ping_id] = event
         return ping_id, event
 
-    def ack(self, ping_id: typing.Union[bytes, bytearray]):
+    def ack(self, ping_id: bytes | bytearray):
         event = self._pings.pop(bytes(ping_id))
         event.set()
